/* eslint-env browser */
(function() {
  'use strict';
  var currentPdf;
  var currentOverview = [];
  var config = {};
  var currentPage = null;
  /**
   * Function to initialize the slider for the min max sliders
   * @param {Object} range is the div html object where slider should appear
   * @param {number} min is the minimal slider value
   * @param {number} max is the maximal slider value
   * @param {int[]} initValues contains the initial min and max value of the slider handles
   */
  function initRangeSlider(range, min, max, initValues){

    noUiSlider.create(range, {
      start: initValues, // 2 handles, starting at...
      connect: true, // Display a colored bar between the handles
      range: {
        'min': min,
        'max': max
      },
      tooltips: true,
      step: 1
    });
  }


  /**
   * This function maps a value of an interval between 0 and 1
   * @param {number} val is the value to map
   * @param {number} min is the minimal interval boundary
   * @param {number} max is the maximal interval boundary
   * @returns {number} a value between 0 and 1
   */
  function normalize(val, min, max){

    var n = (val - min)/(max - min);

    if (n > 1){
      return 1;
    }
    else if (n < 0){
      return 0;
    }else{
      return n;
    }
  }

  /**
   * This function calculates a normalized rating based of a sentence length
   * the used measurement is based on a study https://strainindex.wordpress.com/2008/07/28/the-average-sentence-length/
   * @param {number} sentenceLenth the length of the sentence to get a rating from
   * @returns {number} a rating between 0 and 1
   */
  /*function getSentenceLengthRate(sentenceLength){
      //TODO: refine measurement
      if(sentenceLength <= sentenceLengthScale[0]){
        return 0.0;
      }else if(sentenceLength > sentenceLengthScale[0] && sentenceLength <= sentenceLengthScale[1])
      {
        return normalize(sentenceLength, sentenceLengthScale[0], sentenceLengthScale[1]) * 0.166;
      }else if(sentenceLength > sentenceLengthScale[1] && sentenceLength <= sentenceLengthScale[2])
      {
        return normalize(sentenceLength, sentenceLengthScale[1], sentenceLengthScale[2]) * 0.166*2;
      }else if (sentenceLength > sentenceLengthScale[2] && sentenceLength <= sentenceLengthScale[3]){
        return normalize(sentenceLength, sentenceLengthScale[2], sentenceLengthScale[3]) * 0.166*3;
      }else if (sentenceLength > sentenceLengthScale[3] && sentenceLength <= sentenceLengthScale[4]){
        return normalize(sentenceLength, sentenceLengthScale[3], sentenceLengthScale[4]) * 0.166*4;
      }else if (sentenceLength > sentenceLengthScale[4] && sentenceLength <= sentenceLengthScale[5]){
        return normalize(sentenceLength, sentenceLengthScale[4], sentenceLengthScale[5]) * 0.166*5;
      }else if (sentenceLength > sentenceLengthScale[5] && sentenceLength <= sentenceLengthScale[6]){
        return normalize(sentenceLength, sentenceLengthScale[5], sentenceLengthScale[6]) * 0.166*6;
      }else {
        return 1.0;
      }

  }*/

  /**
   * This function checks the complexity of the structure of the given sentence and returns a complexity ratiing.
   * The complexity of a sentence is based on the number of occured splitting words normalzed by the sentence length.
   * @param {string} sentence the sentence of which to calculate the complexity ratio
   * @returns {number} a rating between 0 and 1
   */
  function checkSentenceStructure(sentence){

    //check if
    var splittingWords = sentence.match(/(\swho\s|\swhich\s|\swhose\s|\sthat\s|\sbecause\s|\sbut\s|\sor\s|\swhen\s|\sif\s|\safter\s|\swhenever\s|,|\s-\s|\(|\)|\[|\]|\{|\})/g);

    return ((splittingWords == null)? 0 : splittingWords.length)

  }

  ////////////////////
  // SENTENCE ANALYSIS
  ////////////////////

   /**
    * This function calculates the different readability factors of a single sentence
    * @param {string} sentence the sentence of which to calculate the readability ratios.
    * @returns {Object} the different measurements how difficult the given sentence is. 0 means very easy, 1 means very difficult.
    */
   function analyzeSentence(sentence) {

      //TODO: filter sentences

     var calcComp = checkSentenceStructure(sentence.str);

     var numberOfNouns = sentence.nlp.nouns().length;
     var numberOfVerbs = sentence.nlp.verbs().length;

     //calculate noun to verb ratio - the higher this value the more difficult to read
     var ntvr = (numberOfNouns > 0) ? numberOfNouns/(numberOfNouns+numberOfVerbs) : 0;

     var sentenceLength = sentence.nlp.wordCount();

     var sumWordLength = 0;
     var sumWordCount = 0;
     var sumEasyWords = 0;
     var sumWordWeight = 0;
     var words = [];

     var wordSlider = document.getElementById("wordlengthSlider").noUiSlider.get();
     var minWord = Number(wordSlider[0]);
     var maxWord = Number(wordSlider[1]);

     config.minWordlength = minWord;
     config.maxWordlength = maxWord;

     for(var i = 0; i < sentence.nlp.list.length; i++){
        var sen = sentence.nlp.list[i];
        sumWordCount += sen.terms.length;
        for(var t = 0; t < sen.terms.length; t++){
            var term = sen.terms[t];
            var text = term.out("text").toLowerCase();
            text = text.replace(/[^a-zA-Z0-9]/g, '').trim();

            var diff = 1.0;

            if(commonWords.includes(text) || extendedVocabulary.includes(text)){
              diff = 0.0;
              sumEasyWords += 1;
            }

            var tlength = text.length;
            var currentWord = new Object();
            currentWord.length = tlength;
            currentWord.complexity = diff;
            currentWord.weight = normalize(tlength, minWord, maxWord);
            words[t] = currentWord;

            sumWordLength += tlength;
            sumWordWeight += currentWord.weight;
        }
     }

     //get the min and max values of the sentence length
     var sentencelengthSlider = document.getElementById("sentencelengthSlider").noUiSlider.get();
     var minSentence = Number(sentencelengthSlider[0]);
     var maxSentence = Number(sentencelengthSlider[1]);
     var len = normalize(sentenceLength, minSentence, maxSentence); //get a weight for a given value

     config.minSentencelength = minSentence;
     config.maxSentencelength = maxSentence;

     //get min and max values of the number of nested sentences
     var sentencelengthSlider = document.getElementById("sentencecomplexitySlider").noUiSlider.get();
     var minSenComp = Number(sentencelengthSlider[0]);
     var maxSenComp = Number(sentencelengthSlider[1]);
     var senComp = normalize(calcComp, minSenComp, maxSenComp);//get a weight for a given value

     config.minSentencecomplexity = minSenComp;
     config.maxSentencecomplexity = maxSenComp;

     var result = {
       wordsProperties: words, //for each word in a sentence, the length and the complexity (if it is an easy word or not) is returned (wordsProperties[0] .. corresponds to the first word in a sentence, wordsProperties[1] is the second etc. wordsProperties[wordCount-1] is the last one of a sentence)
       avgWordWeight: sumWordCount > 0 && sumWordWeight > 0 ? sumWordWeight/sumWordCount : 0,
       wordComplexity: sumEasyWords > 0 && sumWordCount > 0 ? 1-(sumEasyWords/sumWordCount) : 1, //percentage of terms not contained in the 1000 common used words or in specific domain vocabulary
       nominalForm: ntvr, //noun-verb ratio
       sentenceLength: len,
       sentenceComplexity: senComp //branching factor in the phrase structure tree

     };
     result.average = (result.avgWordWeight + result.wordComplexity + result.nominalForm + result.sentenceLength + result.sentenceComplexity)/5;
     return result;
  }

  /**
   * calculates the average readability of a page
   * @param {Object} page
   */
  function analyzePageAverageReadability(page) {
    //Page average readability
    page.readability = {
      avgWordWeight: 0,
      wordComplexity: 0,
      nominalForm: 0,
      sentenceLength: 0,
      sentenceComplexity: 0,
      average: 0,
    };
    for(var i = 0; i < page.sentences.length; i++) {
      page.readability.avgWordWeight += page.sentences[i].readability.avgWordWeight;
      page.readability.wordComplexity += page.sentences[i].readability.wordComplexity;
      page.readability.nominalForm += page.sentences[i].readability.nominalForm;
      page.readability.sentenceLength += page.sentences[i].readability.sentenceLength;
      page.readability.sentenceComplexity += page.sentences[i].readability.sentenceComplexity;
      page.readability.average += page.sentences[i].readability.average;
    }

    page.readability.avgWordWeight /= page.sentences.length;
    page.readability.wordComplexity /= page.sentences.length;
    page.readability.nominalForm /= page.sentences.length;
    page.readability.sentenceLength /= page.sentences.length;
    page.readability.sentenceComplexity /= page.sentences.length;
    page.readability.average /= page.sentences.length;

  }

  /**
   * PDF Import
   **/

  // This function is called after the pdf got imported
  // and the senteces of each page were found
  // Structure:
  // pdf =  {
  //   pages: [
  //     {
  //       page:pdfjsPage,
  //       sentences: [
  //        {nlp: nlpObject, str: sentence as string},
  //        {nlp: nlpObject, str: sentence as string},...]
  //     },
  //     ...
  //   ]
  // }

  function pdfFinishedGettingSentences(pdf) {
    //console.log(pdf);
    currentPdf = pdf;
    renderPageOverview(pdf);
  }

  function analyze() {
    var finalPdf = {}; //PDF with all info
    finalPdf.pages = [];
    var filePdf = document.getElementById("inputPdf");
    filePdf = filePdf.files[0];
    var reader  = new FileReader();

    reader.addEventListener("load", function () {
      PDFJS.getDocument(reader.result).then(function(pdf) {
        var fetchedPageCount = 0;
        for(var i = 1; i <= pdf.numPages; ++i) {
          pdf.getPage(i).then(function(page) {
            var cPage = finalPdf.pages[page.pageIndex] = {};
            cPage.page = page;
            cPage.sentences = [];
            (function(currentPage) { //Pass cPage as closure paramter, to make sure it does not change in promise.
              //Get text content of page
              page.getTextContent().then(function(textContent) {
                var textstr = "";
                var textBlocIndices = [];
                var currentBlocIndex = 0;
                var charsNotPartOfBlocs = [];
                for(var i = 0; i < textContent.items.length; i++) {
                  textBlocIndices[i] = currentBlocIndex;
                  //Split titles with different font
                  if(i > 0 && textContent.items[i].fontName !== textContent.items[i-1].fontName) {

                    if ((i-2) > 0){
                      var prevTextDistance = Math.abs(textContent.items[i-2].transform[5] - textContent.items[i-1].transform[5]);
                      var curTextDistance = Math.abs(textContent.items[i-1].transform[5] - textContent.items[i].transform[5]);

                      //if the current distance is 0 the text is in the same line
                      if(curTextDistance > 0){
                        var diff = Math.abs(prevTextDistance-curTextDistance);

                        if (diff > 0.8){
                          charsNotPartOfBlocs.push(textstr.length);
                          textstr += "\n";
                        }
                      }
                    }else if((i+2)< textContent.items.length){
                      var followingTextDistance = Math.abs(textContent.items[i+2].transform[5] - textContent.items[i+1].transform[5]);
                      var curTextDistance = Math.abs(textContent.items[i+1].transform[5] - textContent.items[i].transform[5]);

                      if(curTextDistance > 0){
                        var diff = Math.abs(followingTextDistance-curTextDistance);

                        if (diff > 0.8){
                          charsNotPartOfBlocs.push(textstr.length);
                          textstr += "\n";
                        }
                      }
                    }
                  }
                  textstr += textContent.items[i].str;
                  charsNotPartOfBlocs.push(textstr.length);
                  textstr += " ";
                  currentBlocIndex += textContent.items[i].str.length;
                }
                var textAnalysis = nlp(textstr);
                var currentSentenceStart = 0;
                for(var i = 0; i < textAnalysis.list.length; i ++) {
                  var sentenceStr = textAnalysis.list[i].out("text");
                  currentPage.sentences[i] = {};
                  currentPage.sentences[i].nlp = nlp(sentenceStr);
                  currentPage.sentences[i].str = sentenceStr.trim();

                  var blocstr = "";
                  for(var j = currentSentenceStart; j < currentSentenceStart+sentenceStr.length; j++) {
                      if(charsNotPartOfBlocs.indexOf(j) == -1)
                        blocstr += textstr.charAt(j);
                  }
                  currentPage.sentences[i].blocstr = blocstr;
                  //currentPage.sentences[i].blocstr = currentPage.sentences[i].blocstr.replace("\n", "");
                  currentPage.sentences[i].readability = analyzeSentence(currentPage.sentences[i]);

                  currentSentenceStart += sentenceStr.length;
                }
                analyzePageAverageReadability(currentPage);

                currentPage.textContent = textContent;
                currentPage.textBlocIndices = textBlocIndices;

                fetchedPageCount++;

                //Received text content of last page
                if(fetchedPageCount == pdf.numPages) {
                  pdfFinishedGettingSentences(finalPdf);
                }
              });
            })(cPage);

          });
        }
      });
    }, false);
    reader.readAsDataURL(filePdf);
  }

  ////////////////////
  // RENDERING STUFF
  ////////////////////

  //HTML templates for the different views
  var initHTML = document.getElementById("tplInit").innerHTML;
  var pagesHTML = document.getElementById("tplPages").innerHTML;
  var pageOverviewItemHTML = document.getElementById("tplPageOverviewItem").innerHTML;
  var pageDetailHTML = document.getElementById("tplPageDetail").innerHTML;
  var sentenceDetailHTML = document.getElementById("tplSentenceDetail").innerHTML;

  //Elements that don't change
  var elmContent = document.getElementById("content");
  var textSizeCalculator = document.getElementById("textwidthcalculator");

  /**
   * Function that calculates the color from a red-white-blue gradient, based on a norm.
   * @param {number} norm The norm to get a color from
   * @returns red, green and blue color values based on the norm(0 = red, 0.5 = white, 1 = blue)
   */
  function getColorForNorm(norm) {
    var color = {};
    if(norm < 0.5) {
      color.r = Math.round(norm * 2 * 255);
      color.g = Math.round(color.r);
      color.b = 255;
    } else {
      color.b = Math.round(((1-norm) * 2) * 255);
      color.g = Math.round(color.b);
      color.r = 255;
    }
    return color;
  }

  /**
   * This function renders a pdf page into a canvas with a given scal
   * @param {Object} page PDFJs page to render
   * @param {number} scale how much to scale the page with
   * @returns {Element} canvas where the pdf page renders on(may not be finished rendering when the function returns)
   */
  function renderPage(page, scale) {
    var viewport = page.getViewport(scale);

    var canvas = document.createElement("canvas");
    var context = canvas.getContext('2d');
    canvas.height = viewport.height;
    canvas.width = viewport.width;

    var renderContext = {
      canvasContext: context,
      viewport: viewport
    };
    page.render(renderContext);
    return canvas;
  }

  /**
   *  Given a page and a character index of the text in the page,
   * this function will return the bloc index that that character is in
   * @param {Object} page The page object from which you have the text character index
   * @param {number} index The character index in the text of the page
   * @returns {number} The index of the bloc the character is in
   */
  function getBlocIndexForTextIndex(page, index) {
    var i = 0;
    while(i < page.textBlocIndices.length &&  page.textBlocIndices[i] <= index) {
      i++;
    }
    //if(i==0) return 0;
    return i - 1;
  }

  /**
   * This function returns the width of a text with a given font
   * @param {Element} canvas A temporary/hidden canvas to draw onto
   * @param {string} text The text to calculate the widh on
   * @param {string} font The name/size of the font
   * @returns {number} the width of the text with the given font
   */
  function getTextWidth(canvas, text, font) {
    var context = canvas.getContext("2d");
    context.font = font;
    var metrics = context.measureText(text);
    return metrics.width;
  }

/*
  function getTextWidth2(canvas, text, font) {
    textSizeCalculator.style.font = font;
    return textSizeCalculator.getBoundingClientRect().width;
  }
*/

/**
 * Draws a filled rectanle inside a document element
 * @param {Element} parent The element in which the rectangle should be drawn in
 * @param {number} x The x coordinate relative to the parent
 * @param {number} y The y coordinate relative to the parent
 * @param {number} width The width of the rectangle
 * @param {number} height The height of the rectangle
 * @param {number} scale Global scale of width and height
 * @param {string} color the fill color. Same as color in css attributes.(ex: rgb(255, 0, 0))
 * @param {string} classname additional css class to give to the element
 */
  function drawRect(parent, x, y, width, height, scale, color, classname) {
    var div = document.createElement("div");
    div.style.width = width * scale + "px";
    div.style.height = height  * scale + "px";
    div.style.left = x * scale  + "px";
    div.style.top = y * scale  + "px";
    div.className = "sentencemarker";
    //div.style.zoom = scale;
    div.style.background = color;
    div.className += " "+ classname;
    //div.innerText = text;
    parent.appendChild(div);
  }

  /**
   *Gets the height of the viewport
   *@returns {number} the height of the viewport
   */
  function getHeight() {
    if (self.innerHeight) {
      return self.innerHeight;
    }

    if (document.documentElement && document.documentElement.clientHeight) {
      return document.documentElement.clientHeight;
    }

    if (document.body) {
      return document.body.clientHeight;
    }
  }

  /**
   * This function renders the detail view of a pdf page.
   * This page consists of the pdf preview with the text highlighted in the
   * respective color and the table with more details about every sentence.
   * To highlight the text parts in the pdf preview,  colored+transparent
   * HTML-divs get rendered over the canvas with the respective position and size.
   * Finding the correct position proved a difficult task, as sentence might span multiple lines and
   * start/end at the middle of the page. As PDFjs only returns coordinates of
   * blocs of texts and not the actual coordinates of sentences, some calculations must be done.
   * A sentence might begin/end in the middle of a bloc, span across multiple blocs or
   * one bloc might even contain multiple sentences. This function handles all
   * these scenarios so that the overlayed divs are positioned correctly.
   * @param {Object} page The pdf page object
   */
  function renderPageDetail(page) {
    currentPage = page;
    elmContent.innerHTML = pageDetailHTML; //Load
    var pagePreview = document.getElementById("pagePreview");
    var sentenceList = document.getElementById("pageText");
    var canvas = renderPage(page.page, 2);

    var pageNext = document.getElementById("nextPage");
    var pagePrevious = document.getElementById("previousPage");

    document.getElementById("btnBackOverview").style.display = "inherit";
    document.getElementById("btnStartPage").style.display = "none";

    document.getElementById("wlTooltip").innerHTML += "<br/> (min: "+config.minWordlength+", max: "+config.maxWordlength+")";
    document.getElementById("slTooltip").innerHTML += "<br/> (min: "+config.minSentencelength+", max: "+config.maxSentencelength+")";
    document.getElementById("scTooltip").innerHTML += "<br/> (min: "+config.minSentencecomplexity+", max: "+config.maxSentencecomplexity+")";



    if(page.page.pageIndex == 0) {
      pagePrevious.style.display = "none";
    } else {
      pagePrevious.addEventListener("click", function() {
        renderPageDetail(currentPdf.pages[page.page.pageIndex -1]);
      });
    }

    if(page.page.pageIndex == currentPdf.pages.length - 1) {
      pageNext.style.display = "none";
    } else {
      pageNext.addEventListener("click", function() {
        renderPageDetail(currentPdf.pages[page.page.pageIndex + 1]);
      });
    }

    //canvas.style.width = "100%";
    canvas.style.height = "100%"; //(getHeight() - 84) + "px";
    canvas.style.top = "84px";
    pagePreview.appendChild(canvas);
    pagePreview.style.width = canvas.style.width;

    var currentSentenceStartIndex = 0;
    var scale = (getHeight() - 84)/canvas.height;
    for(var i = 0; i < page.sentences.length; i ++) {
      var sentence = page.sentences[i];

      ////
      // Highlighting the sentenes in the pdf preview happens here
      ////
      var c = getColorForNorm(sentence.readability.average);
      var color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      var currentSentenceEndIndex = currentSentenceStartIndex + sentence.blocstr.length - 1;

      //TODO Performance improvements: Make getBlocIndexForTextIndex start at last found bloc
      var startBlocIndex = getBlocIndexForTextIndex(page, currentSentenceStartIndex);
      var startIndexInBloc = currentSentenceStartIndex -  page.textBlocIndices[startBlocIndex];
      var startBloc = page.textContent.items[startBlocIndex];

      var endBlocIndex = getBlocIndexForTextIndex(page, currentSentenceEndIndex);
      var endIndexInBloc = currentSentenceEndIndex -  page.textBlocIndices[endBlocIndex];
      var endBloc = page.textContent.items[endBlocIndex];

      var startBlocStyle = page.textContent.styles[startBloc.fontName];
      var startBlocSkipText = startBloc.str.substring(0, startIndexInBloc);
      var startBlocFont = startBlocStyle.fontFamily;
      var startBlocFontSize = startBloc.height;
      var startBlocSkip = getTextWidth(canvas, startBlocSkipText, startBlocFontSize*1000 + "px " + startBlocFont)/1000.0;

      var xStart = startBloc.transform[4] * 2 + startBlocSkip * 2;
      var xWidth = startBloc.width * 2 - startBlocSkip * 2;
      if(startBlocIndex == endBlocIndex) {
        var endBlocSkipText = endBloc.str.substring(endIndexInBloc + 1, endBloc.str.length);
        var endBlocSkip = getTextWidth(canvas, endBlocSkipText, startBlocFontSize*1000 + "px " + startBlocFont)/1000.0;
        xWidth -= endBlocSkip * 2;
      }

      var yStart = canvas.height - startBloc.transform[5] * 2
        - startBloc.height * 2 - startBlocStyle.ascent * 2;
      var yHeight = startBloc.height * 2 + startBlocStyle.ascent * 2
      + (-startBlocStyle.descent) * 2;


      drawRect(pagePreview, xStart, yStart, xWidth, yHeight, scale, color, "sent-"+i);

      for(var j = startBlocIndex + 1; j < endBlocIndex; j++) {
        var middleBloc = page.textContent.items[j];
        var middleBlocStyle = page.textContent.styles[middleBloc.fontName];
        var xStart = middleBloc.transform[4] * 2;
        var xWidth = middleBloc.width * 2
        var yStart = canvas.height - middleBloc.transform[5] * 2
          - middleBloc.height * 2 - middleBlocStyle.ascent * 2;
        var yHeight = middleBloc.height * 2 + middleBlocStyle.ascent * 2
          + (-middleBlocStyle.descent) * 2;
        drawRect(pagePreview, xStart, yStart, xWidth, yHeight, scale, color, "sent-"+i);
      }

      if(startBlocIndex != endBlocIndex) {
        var endBlocStyle = page.textContent.styles[endBloc.fontName];
        var endBlocSkipText = endBloc.str.substring(endIndexInBloc, endBloc.str.length);
        var endBlocFont = endBlocStyle.fontFamily;
        var endBlocFontSize = endBloc.height;
        var endBlocSkip = getTextWidth(canvas, endBlocSkipText, endBlocFontSize*1000 + "px " + endBlocFont)/1000.0;
        var xStart = endBloc.transform[4] * 2;
        var xWidth = endBloc.width * 2
        var yStart = canvas.height - endBloc.transform[5] * 2
          - endBloc.height * 2 - endBlocStyle.ascent * 2;
        var yHeight = endBloc.height * 2 + endBlocStyle.ascent * 2
          + (-endBlocStyle.descent) * 2;
        xWidth -= endBlocSkip * 2;
        drawRect(pagePreview, xStart, yStart, xWidth, yHeight, scale, color, "sent-"+i);


      }
      currentSentenceStartIndex += sentence.blocstr.length;

      ///
      // Table rendering happens here!
      ///

      //Text detail
      var div = document.createElement("tr");
      div.innerHTML = sentenceDetailHTML;
      div.getElementsByClassName("sentence")[0].innerText = sentence.str;
      div.getElementsByClassName("sentence")[0].style.background = color;

      var wl = div.getElementsByClassName("wl")[0];
      wl.innerText = Number((sentence.readability.avgWordWeight).toFixed(2));
      c = getColorForNorm(sentence.readability.avgWordWeight);
      color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      wl.style.background = color;

      var vc = div.getElementsByClassName("vc")[0];
      vc.innerText =  Number((sentence.readability.wordComplexity).toFixed(2));
      c = getColorForNorm(sentence.readability.wordComplexity);
      color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      vc.style.background = color;

      var nf =  div.getElementsByClassName("nf")[0];
      nf.innerText =  Number((sentence.readability.nominalForm).toFixed(2));
      c = getColorForNorm(sentence.readability.nominalForm);
      color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      nf.style.background = color;

      var sl = div.getElementsByClassName("sl")[0];
      sl.innerText =  Number((sentence.readability.sentenceLength).toFixed(2));
      c = getColorForNorm(sentence.readability.sentenceLength);
      color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      sl.style.background = color;

      var sc = div.getElementsByClassName("sc")[0];
      sc.innerText =  Number((sentence.readability.sentenceComplexity).toFixed(2));
      c = getColorForNorm(sentence.readability.sentenceComplexity);
      color = "rgba("+c.r+","+c.g+" ,"+c.b+", 0.5)";
      sc.style.background = color;
      div.sentenceindex = i;
      div.addEventListener("mouseenter", function(e) {
        var line = document.getElementById("line");
        if(line) line.parentNode.removeChild(line);
        var divPos = e.target.getBoundingClientRect();
        var refbloc = document.getElementsByClassName("sent-"+e.target.sentenceindex)[0];
        var endPos = refbloc.getBoundingClientRect();
        line = createLine(endPos.left + refbloc.clientWidth/2, endPos.top + refbloc.clientHeight/2, divPos.left, divPos.top + e.target.clientHeight/2);
        line.id = "line";
        document.body.appendChild(line);
      });
      div.addEventListener("mouseleave", function() {
        var line = document.getElementById("line");
        if(line) line.parentNode.removeChild(line);
      });
      sentenceList.appendChild(div);
    }
  }

  /**
   * This function renders a single page overview item in the overview screen.
   * @param {Object} page the page object which to render in the overview.
   * @returns {Element} the rendered element
   */
  function renderPageOverviewItem(page) {
    var canvas = renderPage(page.page, 0.3);

    var pageOverviewItem = document.createElement("span");
    pageOverviewItem.innerHTML = pageOverviewItemHTML;
    pageOverviewItem.style.borderSpacing = "20px";
    var c = getColorForNorm(page.readability.average);
    pageOverviewItem.firstElementChild.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";
    document.getElementById("btnStartPage").style.display = "inherit";

    var titleContainer = pageOverviewItem.getElementsByClassName("mdl-card__title")[0];
    titleContainer.appendChild(canvas);

    //add click event to canvas object
    pageOverviewItem.getElementsByClassName("mdl-card")[0].addEventListener("click", function(){
      renderPageDetail(page);
    });

    var wordLengthR= pageOverviewItem.getElementsByClassName("wordlengthr")[0];
    var vobabR= pageOverviewItem.getElementsByClassName("vocabr")[0];
    var nominalR= pageOverviewItem.getElementsByClassName("nominalr")[0];
    var sentenceLengthR= pageOverviewItem.getElementsByClassName("sentencelengthr")[0];
    var sentenceComplexR= pageOverviewItem.getElementsByClassName("sentencecomplexr")[0];
    wordLengthR.innerText = Number((page.readability.avgWordWeight).toFixed(2));
    vobabR.innerText =  Number((page.readability.wordComplexity).toFixed(2));
    nominalR.innerText =  Number((page.readability.nominalForm).toFixed(2));
    sentenceLengthR.innerText =  Number((page.readability.sentenceLength).toFixed(2));
    sentenceComplexR.innerText =  Number((page.readability.sentenceComplexity).toFixed(2));
    c = getColorForNorm(page.readability.avgWordWeight);
    wordLengthR.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";
    c = getColorForNorm(page.readability.wordComplexity);
    vobabR.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";
    c = getColorForNorm(page.readability.nominalForm);
    nominalR.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";
    c = getColorForNorm(page.readability.sentenceLength);
    sentenceLengthR.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";
    c = getColorForNorm(page.readability.sentenceComplexity);
    sentenceComplexR.style.background = "rgba("+c.r+", "+c.g+", "+c.b+", 0.5)";

    return pageOverviewItem;
  }

  /**
   * renders the pages of a given pdf file in overview screen
   * @param {Object} pdf Object that stores the information stored in the loaded pdf
   */
  function renderPageOverview(pdf) {
    currentPage = null;
    document.getElementById("btnBackOverview").style.display = "none";
    elmContent.innerHTML = pagesHTML;
    var pageGrid = document.getElementById("pagegrid");
    pdf.pages.forEach(function(page) {
      var pageItem = renderPageOverviewItem(page);
      pageGrid.appendChild(pageItem);
    });
  }

  /**
   * calls the function to render the overview of the pdf pages
   */
  function showOverview() {
    renderPageOverview(currentPdf);
  }

  /**
  * Returns all text from a pdf
  */
   function getPageText(pageNum, PDFDocumentInstance) {
    // Return a Promise that is solved once the text of the page is retrieven
    return new Promise(function (resolve, reject) {
        PDFDocumentInstance.getPage(pageNum).then(function (pdfPage) {
            // The main trick to obtain the text of the PDF page, use the getTextContent method
            pdfPage.getTextContent().then(function (textContent) {
                var textItems = textContent.items;
                var finalString = "";

                // Concatenate the string of the item to the final string
                for (var i = 0; i < textItems.length; i++) {
                    var item = textItems[i];

                    finalString += item.str + " ";
                }

                // Solve promise with the text retrieven from the page
                resolve(finalString);
            });
        });
    });
  }


  /**
  * Extracts vocabulary out of a reference paper
  */
  function extractVocabulary(e){
    var filePdf = document.getElementById("referencePdf");
     filePdf = filePdf.files[0];
     var reader  = new FileReader();


    reader.addEventListener("load", function(){

      PDFJS.getDocument(reader.result).then(function(pdf) {

      var pagesPromises = [];

      for (var i = 0; i < pdf.numPages; i++) {
          // Required to prevent that i is always the total of pages
          (function (pageNumber) {
              // Store the promise of getPageText that returns the text of a page
              pagesPromises.push(getPageText(pageNumber, pdf));
          })(i + 1);
      }

      // Execute all the promises
      Promise.all(pagesPromises).then(function (pagesText) {

          // Display text of all the pages in the console
          // e.g ["Text content page 1", "Text content page 2", "Text content page 3" ... ]
          var result = pagesText.toString();
        // split string by spaces (including spaces, tabs, and newlines)
          var wordsArray = result.split(/\s+/);
          var dictionary = new Map();

          for(var i = 0; i < wordsArray.length; i++){
            var key = wordsArray[i].toLowerCase();
            key = key.replace(/[^a-zA-Z0-9]/g, '').trim();

            if(!commonWords.includes(key) && key != ""){
              if(dictionary.has(key)){
                dictionary.set(key, dictionary.get(key)+1);
              }
              else{
                dictionary.set(key, 1);
              }
            }
          }


      var a = [];
      for(var x of dictionary)
        a.push(x);

      a.sort(function(x, y) {
        return y[1]-x[1];
      });


      extendedVocabulary = a.length > 1000 ? a.slice(0, 1000) : a;

      });
      }, function (reason) {
      // PDF loading error
      console.error(reason);
   });
    }, false);

    reader.readAsDataURL(filePdf);

  }

  /**
   * Shows an error message at the bottom of the screen
   * @param {string} message the error message to show
   */
  function showErrorWithMessage(message) {
    var data = {message: message};
    document.getElementById("error").MaterialSnackbar.showSnackbar(data);
  }

  /**
   * call this function on the start screen, when the user finished providing information.
   */
  function startAnalysis(){
    var filePdf = document.getElementById("inputPdf");
    if(filePdf.files.length > 0) {
      document.getElementById("btnAnalyze").setAttribute("disabled", true);
      document.getElementById("btnAnalyze").innerText = "Please wait, loading...";
      analyze();
    } else {
      showErrorWithMessage("Please Upload a PDF first");
    }

  }

  /**
   * Initializes the accorion on the start screen
   */
  function initAccordion(){
    var acc = document.getElementsByClassName("accordion");
    var i;

    for (i = 0; i < acc.length; i++) {
      acc[i].onclick = function() {
        this.classList.toggle("active");
        var panel = this.nextElementSibling;
        if (panel.style.maxHeight){
          panel.style.maxHeight = null;
        } else {
          panel.style.maxHeight = panel.scrollHeight + "px";
        }
      }
    }
  }

  /**
   * This function get called when the page is first loaded, or whenever the user comes back to the start screen.
   * It resets all the components back to its initial values.
   */
  function init() {
    window.extendedVocabulary = [];
    elmContent.innerHTML = initHTML;
    document.getElementById("btnBackOverview").style.display = "none";
    document.getElementById("btnStartPage").style.display = "none";
    document.getElementById("btnAnalyze").addEventListener("click", startAnalysis);
    document.getElementById("referencePdf").addEventListener("change", extractVocabulary);
    document.getElementById("btnBackOverview").addEventListener("click", showOverview);
    document.getElementById("btnStartPage").addEventListener("click", init);
    document.getElementById("btnAnalyze").removeAttribute("disabled");
    document.getElementById("btnAnalyze").innerText = "Analyze the text";
    document.getElementById("title").addEventListener("click", init);

    initRangeSlider(document.getElementById('wordlengthSlider'), 1, 35, [3, 10]);
    initRangeSlider(document.getElementById('sentencelengthSlider'), 1, 50, [8, 29]);
    initRangeSlider(document.getElementById('sentencecomplexitySlider'), 0, 15, [1, 6]);
    initAccordion();
  }

  /**
   * This function gets called whenever the user resizes the window
   */
  function resize() {
    if(currentPage) {
      renderPageDetail(currentPage);
    }
  }

  /**
   * Creates a line with a given start position, length and angle
   * @param {number} x x start coordinate of the line
   * @param {number} y y start coordinate of the line
   * @param {number} length length of the line
   * @param {number} angle orientation of the line
   * @returns {Element} an element that renders as a line
   */
  function createLineElement(x, y, length, angle) {
      var line = document.createElement("div");
      var styles = 'border: 1px solid rgba(0,0,0,0.5); '
                 + 'width: ' + length + 'px; '
                 + 'height: 0px; '
                 + '-moz-transform: rotate(' + angle + 'rad); '
                 + '-webkit-transform: rotate(' + angle + 'rad); '
                 + '-o-transform: rotate(' + angle + 'rad); '
                 + '-ms-transform: rotate(' + angle + 'rad); '
                 + 'position: absolute; '
                 + 'top: ' + y + 'px; '
                 + 'left: ' + x + 'px; '
                 + 'z-index: 9999; ';
      line.setAttribute('style', styles);
      return line;
  }

  /**
   * Creates a line with a given start and end position
   * @param {number} x1 x start coordinate of the line
   * @param {number} y1 y start coordinate of the line
   * @param {number} x2 x end coordinate of the line
   * @param {number} yy y end coordinate of the line
   * @returns {Element} an element that renders as a line
   */
  function createLine(x1, y1, x2, y2) {
      var a = x1 - x2,
          b = y1 - y2,
          c = Math.sqrt(a * a + b * b);

      var sx = (x1 + x2) / 2,
          sy = (y1 + y2) / 2;

      var x = sx - c / 2,
          y = sy;

      var alpha = Math.PI - Math.atan2(-b, a);

      return createLineElement(x, y, c, alpha);
  }

  init();
  window.onresize = function(event) {
    resize();
  }

})();
